@@ -18,11 +18,11 @@
 package org.apache.kyuubi.engine.spark.operation
 
 import org.apache.kyuubi.engine.spark.WithSparkSQLEngine
-import org.apache.kyuubi.operation.BasicDeltaJDBCTests
+import org.apache.kyuubi.operation.DeltaMetadataTests
 import org.apache.kyuubi.tags.DeltaTest
 
 @DeltaTest
-class SparkDeltaOperationSuite extends WithSparkSQLEngine with BasicDeltaJDBCTests {
+class SparkDeltaOperationSuite extends WithSparkSQLEngine with DeltaMetadataTests {
 
   override protected def jdbcUrl: String = getJdbcUrl
 
