@@ -18,11 +18,11 @@
 package org.apache.kyuubi.engine.spark.operation
 
 import org.apache.kyuubi.engine.spark.WithSparkSQLEngine
-import org.apache.kyuubi.operation.BasicIcebergJDBCTests
+import org.apache.kyuubi.operation.IcebergMetadataTests
 import org.apache.kyuubi.tags.IcebergTest
 
 @IcebergTest
-class SparkIcebergOperationSuite extends WithSparkSQLEngine with BasicIcebergJDBCTests {
+class SparkIcebergOperationSuite extends WithSparkSQLEngine with IcebergMetadataTests {
 
   override protected def jdbcUrl: String = getJdbcUrl
 
