@@ -17,26 +17,29 @@
 
 package org.apache.kyuubi.engine.spark.operation
 
-import java.sql.{DatabaseMetaData, ResultSet, SQLFeatureNotSupportedException}
+import java.sql.{DatabaseMetaData, ResultSet, SQLException, SQLFeatureNotSupportedException}
 
 import scala.collection.JavaConverters._
 import scala.util.Random
 
-import org.apache.hive.common.util.HiveVersionInfo
-import org.apache.hive.service.cli.HiveSQLException
+import org.apache.hadoop.hdfs.security.token.delegation.{DelegationTokenIdentifier => HDFSTokenIdent}
+import org.apache.hadoop.hive.thrift.{DelegationTokenIdentifier => HiveTokenIdent}
+import org.apache.hadoop.io.Text
+import org.apache.hadoop.security.{Credentials, UserGroupInformation}
+import org.apache.hadoop.security.token.{Token, TokenIdentifier}
 import org.apache.hive.service.rpc.thrift._
-import org.apache.hive.service.rpc.thrift.TCLIService.Iface
-import org.apache.hive.service.rpc.thrift.TOperationState._
+import org.apache.spark.kyuubi.SparkContextHelper
 import org.apache.spark.sql.catalyst.analysis.FunctionRegistry
 import org.apache.spark.sql.types._
 
 import org.apache.kyuubi.Utils
 import org.apache.kyuubi.engine.spark.WithSparkSQLEngine
 import org.apache.kyuubi.engine.spark.shim.SparkCatalogShim
-import org.apache.kyuubi.operation.HiveJDBCTests
+import org.apache.kyuubi.operation.{HiveMetadataTests, SparkQueryTests}
 import org.apache.kyuubi.operation.meta.ResultSetSchemaConstant._
+import org.apache.kyuubi.util.KyuubiHadoopUtils
 
-class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
+class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with SparkQueryTests {
 
   override protected def jdbcUrl: String = getJdbcUrl
   override def withKyuubiConf: Map[String, String] = Map.empty
@@ -78,7 +81,7 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
 
     val ddl =
       s"""
-         |CREATE TABLE IF NOT EXISTS $dftSchema.$tableName (
+         |CREATE TABLE IF NOT EXISTS $defaultSchema.$tableName (
          |  ${schema.toDDL}
          |)
          |USING parquet""".stripMargin
@@ -89,18 +92,34 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
       val metaData = statement.getConnection.getMetaData
 
       Seq("%", null, ".*", "c.*") foreach { columnPattern =>
-        val rowSet = metaData.getColumns("", dftSchema, tableName, columnPattern)
+        val rowSet = metaData.getColumns("", defaultSchema, tableName, columnPattern)
 
         import java.sql.Types._
-        val expectedJavaTypes = Seq(BOOLEAN, TINYINT, SMALLINT, INTEGER, BIGINT, FLOAT, DOUBLE,
-          DECIMAL, DECIMAL, VARCHAR, ARRAY, ARRAY, JAVA_OBJECT, DATE, TIMESTAMP, STRUCT, BINARY,
+        val expectedJavaTypes = Seq(
+          BOOLEAN,
+          TINYINT,
+          SMALLINT,
+          INTEGER,
+          BIGINT,
+          FLOAT,
+          DOUBLE,
+          DECIMAL,
+          DECIMAL,
+          VARCHAR,
+          ARRAY,
+          ARRAY,
+          JAVA_OBJECT,
+          DATE,
+          TIMESTAMP,
+          STRUCT,
+          BINARY,
           STRUCT)
 
         var pos = 0
 
         while (rowSet.next()) {
           assert(rowSet.getString(TABLE_CAT) === SparkCatalogShim.SESSION_CATALOG)
-          assert(rowSet.getString(TABLE_SCHEM) === dftSchema)
+          assert(rowSet.getString(TABLE_SCHEM) === defaultSchema)
           assert(rowSet.getString(TABLE_NAME) === tableName)
           assert(rowSet.getString(COLUMN_NAME) === schema(pos).name)
           assert(rowSet.getInt(DATA_TYPE) === expectedJavaTypes(pos))
@@ -206,7 +225,7 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
       val apis = Seq(metaData.getFunctions _, metaData.getProcedures _)
       Seq("to_timestamp", "date_part", "lpad", "date_format", "cos", "sin").foreach { func =>
         apis.foreach { apiFunc =>
-          val resultSet = apiFunc("", dftSchema, func)
+          val resultSet = apiFunc("", defaultSchema, func)
           while (resultSet.next()) {
             val exprInfo = FunctionRegistry.expressions(func)._1
             assert(resultSet.getString(FUNCTION_CAT).isEmpty)
@@ -399,13 +418,14 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
 
       import org.apache.kyuubi.KYUUBI_VERSION
       assert(metaData.allTablesAreSelectable)
-      assert(metaData.getDatabaseProductName === "Spark SQL")
+      assert(metaData.getDatabaseProductName === "Apache Kyuubi (Incubating)")
       assert(metaData.getDatabaseProductVersion === KYUUBI_VERSION)
-      assert(metaData.getDriverName === "Hive JDBC")
-      assert(metaData.getDriverVersion === HiveVersionInfo.getVersion)
+      assert(metaData.getDriverName === "Kyuubi Project Hive JDBC Shaded Client")
+      assert(metaData.getDriverVersion === KYUUBI_VERSION)
       assert(metaData.getDatabaseMajorVersion === Utils.majorVersion(KYUUBI_VERSION))
       assert(metaData.getDatabaseMinorVersion === Utils.minorVersion(KYUUBI_VERSION))
-      assert(metaData.getIdentifierQuoteString === " ",
+      assert(
+        metaData.getIdentifierQuoteString === " ",
         "This method returns a space \" \" if identifier quoting is not supported")
       assert(metaData.getNumericFunctions === "")
       assert(metaData.getStringFunctions === "")
@@ -449,9 +469,9 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
       assert(metaData.getDefaultTransactionIsolation === java.sql.Connection.TRANSACTION_NONE)
       assert(!metaData.supportsTransactions)
       assert(!metaData.getProcedureColumns("", "%", "%", "%").next())
-      intercept[HiveSQLException](metaData.getPrimaryKeys("", "default", ""))
+      intercept[SQLException](metaData.getPrimaryKeys("", "default", ""))
       assert(!metaData.getImportedKeys("", "default", "").next())
-      intercept[HiveSQLException] {
+      intercept[SQLException] {
         metaData.getCrossReference("", "default", "src", "", "default", "src2")
       }
       assert(!metaData.getIndexInfo("", "default", "src", true, true).next())
@@ -488,18 +508,10 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
     }
   }
 
-  private def waitForOperationToComplete(client: Iface, op: TOperationHandle): Unit = {
-    val req = new TGetOperationStatusReq(op)
-    var state = client.GetOperationStatus(req).getOperationState
-    while (state == INITIALIZED_STATE || state == PENDING_STATE || state == RUNNING_STATE) {
-      state = client.GetOperationStatus(req).getOperationState
-    }
-
-  }
   test("basic open | execute | close") {
     withThriftClient { client =>
-      val operationManager = engine.backendService.sessionManager.
-        operationManager.asInstanceOf[SparkSQLOperationManager]
+      val operationManager =
+        engine.backendService.sessionManager.operationManager.asInstanceOf[SparkSQLOperationManager]
       assert(operationManager.getOpenSparkSessionCount === 0)
 
       val req = new TOpenSessionReq()
@@ -589,7 +601,8 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
       req.setUsername("kentyao")
       req.setPassword("anonymous")
       val queue = "spark.yarn.queue"
-      val conf = Map("use:database" -> "default",
+      val conf = Map(
+        "use:database" -> "default",
         "spark.sql.globalTempDatabase" -> "temp",
         queue -> "new",
         s"set:hiveconf:$queue" -> "newnew",
@@ -761,4 +774,113 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveJDBCTests {
       assert(tFetchResultsResp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
     }
   }
+
+  test("send credentials by TRenewDelegationTokenReq") {
+    // Simulate a secured SparkSQLEngine's credentials
+    val currentTime = System.currentTimeMillis()
+    val hdfsTokenAlias = new Text("HDFS1")
+    val hiveTokenAlias = new Text("hive.server2.delegation.token")
+    val creds1 = createCredentials(currentTime, hdfsTokenAlias.toString, hiveTokenAlias.toString)
+    // SparkSQLEngine may have token alias unknown to Kyuubi Server
+    val unknownTokenAlias = new Text("UNKNOWN")
+    val unknownToken = newHDFSToken(currentTime)
+    creds1.addToken(unknownTokenAlias, unknownToken)
+    SparkContextHelper.updateDelegationTokens(spark.sparkContext, creds1)
+
+    val metastoreUris = "thrift://localhost:9083,thrift://localhost:9084"
+
+    whenMetaStoreURIsSetTo(metastoreUris) { uris =>
+      withThriftClient { client =>
+        val req = new TOpenSessionReq()
+        req.setUsername("username")
+        req.setPassword("password")
+        val tOpenSessionResp = client.OpenSession(req)
+
+        def sendCredentials(client: TCLIService.Iface, credentials: Credentials): Unit = {
+          val renewDelegationTokenReq = new TRenewDelegationTokenReq()
+          renewDelegationTokenReq.setSessionHandle(tOpenSessionResp.getSessionHandle)
+          renewDelegationTokenReq.setDelegationToken(
+            KyuubiHadoopUtils.encodeCredentials(credentials))
+          val renewDelegationTokenResp = client.RenewDelegationToken(renewDelegationTokenReq)
+          assert(renewDelegationTokenResp.getStatus.getStatusCode == TStatusCode.SUCCESS_STATUS)
+        }
+
+        // Send new credentials
+        val creds2 = createCredentials(currentTime + 1, hdfsTokenAlias.toString, uris)
+        // Kyuubi Server may have extra HDFS and Hive delegation tokens
+        val extraHDFSToken = newHDFSToken(currentTime + 1)
+        creds2.addToken(new Text("HDFS2"), extraHDFSToken)
+        sendCredentials(client, creds2)
+        // SparkSQLEngine's tokens should be updated
+        var engineCredentials = UserGroupInformation.getCurrentUser.getCredentials
+        assert(engineCredentials.getToken(hdfsTokenAlias) == creds2.getToken(hdfsTokenAlias))
+        assert(
+          engineCredentials.getToken(hiveTokenAlias) == creds2.getToken(new Text(metastoreUris)))
+        // Unknown tokens should not be updated
+        assert(engineCredentials.getToken(unknownTokenAlias) == unknownToken)
+
+        // Send old credentials
+        val creds3 = createCredentials(currentTime, hdfsTokenAlias.toString, metastoreUris)
+        sendCredentials(client, creds3)
+        // SparkSQLEngine's tokens should not be updated
+        engineCredentials = UserGroupInformation.getCurrentUser.getCredentials
+        assert(engineCredentials.getToken(hdfsTokenAlias) == creds2.getToken(hdfsTokenAlias))
+        assert(
+          engineCredentials.getToken(hiveTokenAlias) == creds2.getToken(new Text(metastoreUris)))
+
+        // No matching tokens
+        val creds4 = createCredentials(currentTime + 2, "HDFS2", "thrift://localhost:9085")
+        sendCredentials(client, creds4)
+        // No token is updated
+        engineCredentials = UserGroupInformation.getCurrentUser.getCredentials
+        assert(engineCredentials.getToken(hdfsTokenAlias) == creds2.getToken(hdfsTokenAlias))
+        assert(
+          engineCredentials.getToken(hiveTokenAlias) == creds2.getToken(new Text(metastoreUris)))
+      }
+    }
+  }
+
+  private def whenMetaStoreURIsSetTo(uris: String)(func: String => Unit): Unit = {
+    val conf = spark.sparkContext.hadoopConfiguration
+    val origin = conf.get("hive.metastore.uris", "")
+    conf.set("hive.metastore.uris", uris)
+    try func.apply(uris)
+    finally {
+      conf.set("hive.metastore.uris", origin)
+    }
+  }
+
+  private def createCredentials(
+      issueDate: Long,
+      hdfsTokenAlias: String,
+      hiveTokenAlias: String): Credentials = {
+    val credentials = new Credentials()
+    credentials.addToken(new Text(hdfsTokenAlias), newHDFSToken(issueDate))
+    credentials.addToken(new Text(hiveTokenAlias), newHiveToken(issueDate))
+    credentials
+  }
+
+  private def newHDFSToken(issueDate: Long): Token[TokenIdentifier] = {
+    val who = new Text("who")
+    val tokenId = new HDFSTokenIdent(who, who, who)
+    tokenId.setIssueDate(issueDate)
+    newToken(tokenId)
+  }
+
+  private def newHiveToken(issueDate: Long): Token[TokenIdentifier] = {
+    val who = new Text("who")
+    val tokenId = new HiveTokenIdent(who, who, who)
+    tokenId.setIssueDate(issueDate)
+    newToken(tokenId)
+  }
+
+  private def newToken(tokeIdent: TokenIdentifier): Token[TokenIdentifier] = {
+    val token = new Token[TokenIdentifier]
+    token.setID(tokeIdent.getBytes)
+    token.setKind(tokeIdent.getKind)
+    val bytes = new Array[Byte](128)
+    Random.nextBytes(bytes)
+    token.setPassword(bytes)
+    token
+  }
 }
