@@ -18,15 +18,19 @@
 package org.apache.kyuubi.engine
 
 import java.io.{File, IOException}
+import java.lang.ProcessBuilder.Redirect
 import java.nio.charset.StandardCharsets
 import java.nio.file.{Files, Path}
 
 import scala.collection.JavaConverters._
+import scala.util.matching.Regex
 
+import com.google.common.collect.EvictingQueue
 import org.apache.commons.lang3.StringUtils.containsIgnoreCase
 
 import org.apache.kyuubi.{KyuubiSQLException, Logging}
 import org.apache.kyuubi.config.KyuubiConf
+import org.apache.kyuubi.operation.log.OperationLog
 import org.apache.kyuubi.util.NamedThreadFactory
 
 trait ProcBuilder {
@@ -49,6 +53,8 @@ trait ProcBuilder {
 
   protected def env: Map[String, String] = conf.getEnvs
 
+  protected val extraEngineLog: Option[OperationLog]
+
   protected val workingDir: Path
 
   final lazy val processBuilder: ProcessBuilder = {
@@ -59,11 +65,14 @@ trait ProcBuilder {
     pb.directory(workingDir.toFile)
     pb.redirectError(engineLog)
     pb.redirectOutput(engineLog)
+    extraEngineLog.foreach(_.addExtraLog(engineLog.toPath))
     pb
   }
 
   @volatile private var error: Throwable = UNCAUGHT_ERROR
-  @volatile private var lastRowOfLog: String = "unknown"
+
+  private val engineLogMaxLines = conf.get(KyuubiConf.SESSION_ENGINE_STARTUP_MAX_LOG_LINES)
+  private val lastRowsOfLog: EvictingQueue[String] = EvictingQueue.create(engineLogMaxLines)
   // Visible for test
   @volatile private[kyuubi] var logCaptureThreadReleased: Boolean = true
   private var logCaptureThread: Thread = _
@@ -74,11 +83,12 @@ trait ProcBuilder {
     val processLogPath = workingDir
     val totalExistsFile = processLogPath.toFile.listFiles { (_, name) => name.startsWith(module) }
     val sorted = totalExistsFile.sortBy(_.getName.split("\\.").last.toInt)
-    val nextIndex = if (sorted.isEmpty) {
-      0
-    } else {
-      sorted.last.getName.split("\\.").last.toInt + 1
-    }
+    val nextIndex =
+      if (sorted.isEmpty) {
+        0
+      } else {
+        sorted.last.getName.split("\\.").last.toInt + 1
+      }
     val file = sorted.find(_.lastModified() < currentTime - engineLogTimeout)
       .map { existsFile =>
         try {
@@ -129,7 +139,7 @@ trait ProcBuilder {
 
               error = KyuubiSQLException(sb.toString() + s"\n See more: $engineLog")
             } else if (line != null) {
-              lastRowOfLog = line
+              lastRowsOfLog.add(line)
             }
           } else {
             Thread.sleep(300)
@@ -150,6 +160,29 @@ trait ProcBuilder {
     proc
   }
 
+  val YARN_APP_NAME_REGEX: Regex = "application_\\d+_\\d+".r
+
+  def killApplication(line: String = lastRowsOfLog.toArray.mkString("\n")): String =
+    YARN_APP_NAME_REGEX.findFirstIn(line) match {
+      case Some(appId) =>
+        env.get(KyuubiConf.KYUUBI_HOME) match {
+          case Some(kyuubiHome) =>
+            val pb = new ProcessBuilder("/bin/sh", s"$kyuubiHome/bin/stop-application.sh", appId)
+            pb.environment()
+              .putAll(env.asJava)
+            pb.redirectError(Redirect.appendTo(engineLog))
+            pb.redirectOutput(Redirect.appendTo(engineLog))
+            val process = pb.start()
+            process.waitFor() match {
+              case id if id != 0 => s"Failed to kill Application $appId, please kill it manually. "
+              case _ => s"Killed Application $appId successfully. "
+            }
+          case None =>
+            s"KYUUBI_HOME is not set! Failed to kill Application $appId, please kill it manually."
+        }
+      case None => ""
+    }
+
   def close(): Unit = {
     if (logCaptureThread != null) {
       logCaptureThread.interrupt()
@@ -163,7 +196,8 @@ trait ProcBuilder {
     error match {
       case UNCAUGHT_ERROR =>
         KyuubiSQLException(s"Failed to detect the root cause, please check $engineLog at server " +
-          s"side if necessary. The last line log is: $lastRowOfLog")
+          s"side if necessary. The last $engineLogMaxLines line(s) of log are:\n" +
+          s"${lastRowsOfLog.toArray.mkString("\n")}")
       case other => other
     }
   }
