@@ -17,13 +17,10 @@
 
 package org.apache.kyuubi.operation
 
-import org.apache.kyuubi.client.KyuubiSyncThriftClient
 import org.apache.kyuubi.session.Session
 
-class GetTableTypes(
-    session: Session,
-    client: KyuubiSyncThriftClient)
-  extends KyuubiOperation(OperationType.GET_TABLE_TYPES, session, client) {
+class GetTableTypes(session: Session)
+  extends KyuubiOperation(OperationType.GET_TABLE_TYPES, session) {
 
   override protected def runInternal(): Unit = {
     try {
