@@ -22,21 +22,21 @@ import java.io.IOException
 import com.codahale.metrics.MetricRegistry
 import org.apache.commons.lang3.StringUtils
 import org.apache.hive.service.rpc.thrift._
+import org.apache.thrift.TException
 import org.apache.thrift.transport.TTransportException
 
-import org.apache.kyuubi.KyuubiSQLException
-import org.apache.kyuubi.client.KyuubiSyncThriftClient
+import org.apache.kyuubi.{KyuubiSQLException, Utils}
 import org.apache.kyuubi.metrics.MetricsConstants.STATEMENT_FAIL
 import org.apache.kyuubi.metrics.MetricsSystem
 import org.apache.kyuubi.operation.FetchOrientation.FetchOrientation
 import org.apache.kyuubi.operation.OperationType.OperationType
-import org.apache.kyuubi.session.Session
+import org.apache.kyuubi.session.{KyuubiSessionImpl, Session}
 import org.apache.kyuubi.util.ThriftUtils
 
-abstract class KyuubiOperation(
-    opType: OperationType,
-    session: Session,
-    client: KyuubiSyncThriftClient) extends AbstractOperation(opType, session) {
+abstract class KyuubiOperation(opType: OperationType, session: Session)
+  extends AbstractOperation(opType, session) {
+
+  protected[operation] lazy val client = session.asInstanceOf[KyuubiSessionImpl].client
 
   @volatile protected var _remoteOpHandle: TOperationHandle = _
 
@@ -56,18 +56,20 @@ abstract class KyuubiOperation(
           MetricsSystem.tracing {
             _.incCount(MetricRegistry.name(STATEMENT_FAIL, errorType))
           }
-          setState(OperationState.ERROR)
           val ke = e match {
             case kse: KyuubiSQLException => kse
-            case te: TTransportException if te.getType == TTransportException.END_OF_FILE &&
-                StringUtils.isEmpty(te.getMessage) =>
+            case te: TTransportException
+                if te.getType == TTransportException.END_OF_FILE &&
+                  StringUtils.isEmpty(te.getMessage) =>
               // https://issues.apache.org/jira/browse/THRIFT-4858
               KyuubiSQLException(
-                s"Error $action $opType: Socket for ${session.handle} is closed", e)
-            case _ =>
-              KyuubiSQLException(s"Error $action $opType: ${e.getMessage}", e)
+                s"Error $action $opType: Socket for ${session.handle} is closed",
+                e)
+            case e =>
+              KyuubiSQLException(s"Error $action $opType: ${Utils.stringifyException(e)}", e)
           }
           setOperationException(ke)
+          setState(OperationState.ERROR)
           throw ke
         }
       }
@@ -86,27 +88,37 @@ abstract class KyuubiOperation(
     }
   }
 
-  override def cancel(): Unit = {
-    if (_remoteOpHandle != null && !isClosedOrCanceled) {
-      try {
-        client.cancelOperation(_remoteOpHandle)
-        setState(OperationState.CANCELED)
-      } catch onError("cancelling")
+  override def cancel(): Unit = state.synchronized {
+    if (!isClosedOrCanceled) {
+      setState(OperationState.CANCELED)
+      if (_remoteOpHandle != null) {
+        try {
+          client.cancelOperation(_remoteOpHandle)
+        } catch {
+          case e @ (_: TException | _: KyuubiSQLException) =>
+            warn(s"Error cancelling ${_remoteOpHandle.getOperationId}: ${e.getMessage}", e)
+        }
+      }
     }
   }
 
-  override def close(): Unit = {
-    if (_remoteOpHandle != null && !isClosedOrCanceled) {
-      try {
-        getOperationLog.foreach(_.close())
-      } catch {
-        case e: IOException =>
-          error(e.getMessage, e)
+  override def close(): Unit = state.synchronized {
+    if (!isClosedOrCanceled) {
+      setState(OperationState.CLOSED)
+      if (_remoteOpHandle != null) {
+        try {
+          getOperationLog.foreach(_.close())
+        } catch {
+          case e: IOException => error(e.getMessage, e)
+        }
+
+        try {
+          client.closeOperation(_remoteOpHandle)
+        } catch {
+          case e @ (_: TException | _: KyuubiSQLException) =>
+            warn(s"Error closing ${_remoteOpHandle.getOperationId}: ${e.getMessage}", e)
+        }
       }
-      try {
-        client.closeOperation(_remoteOpHandle)
-        setState(OperationState.CLOSED)
-      } catch onError("closing")
     }
   }
 
